@@ -1,4 +1,4 @@
-# Copyright © 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2012, 2013, 2014 Jakub Wilk <jwilk@jwilk.net>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the “Software”), to deal
@@ -18,8 +18,12 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+'''
+checks
+'''
+
+import abc
 import collections
-import contextlib
 import difflib
 import email.utils
 import os
@@ -29,11 +33,12 @@ import urllib.parse
 
 import polib
 
+from . import encodings as encinfo
 from . import gettext
 from . import ling
 from . import misc
 from . import polib4us
-from . import rfc2606
+from . import domains
 from . import tags
 
 class EnvironmentNotPatched(RuntimeError):
@@ -43,13 +48,14 @@ class EnvironmentAlreadyPatched(RuntimeError):
     pass
 
 find_unusual_characters = re.compile(
-    r'[\x00-\x08\x0b-\x1a\x1c-\x1f]'  # C0 except TAB, LF, ESC
-    r'|\x1b(?!\[)'  # ESC, except when followed by [
-    r'|\x7f'  # DEL
-    r'|[\x80-\x9f]'  # C1
-     '|\ufffd'  # REPLACEMENT CHARACTER
-     '|[\ufffe\uffff]'  # non-characters
-    r'|(?<=\w)\xbf'  # INVERTED QUESTION MARK but only directly after a letter
+    r'[\x00-\x08\x0B-\x1A\x1C-\x1F]'  # C0 except TAB, LF, ESC
+    r'|\x1B(?!\[)'  # ESC, except when followed by [
+    r'|\x7F'  # DEL
+    r'|[\x80-\x9F]'  # C1
+     '|\uFEFF'  # ZERO WIDTH NO-BREAK SPACE
+     '|\uFFFD'  # REPLACEMENT CHARACTER
+     '|[\uFFFE\uFFFF]'  # non-characters
+    r'|(?<=\w)\xBF'  # INVERTED QUESTION MARK but only directly after a letter
 ).findall
 
 header_fields_with_dedicated_checks = set()
@@ -60,24 +66,17 @@ def checks_header_fields(*fields):
     header_fields_with_dedicated_checks.update(fields)
     return identity
 
-class Checker(object):
+class Checker(object, metaclass=abc.ABCMeta):
 
     _patched_environment = None
 
     @classmethod
-    @contextlib.contextmanager
-    def patched_environment(cls, encinfo):
+    def patch_environment(cls):
         if cls._patched_environment is not None:
             raise EnvironmentAlreadyPatched
-        cls._patched_environment = False
-        with polib4us.patched():
-            with encinfo.extra_encodings():
-                cls._patched_environment = True
-                try:
-                    yield
-                finally:
-                    cls._patched_environment = False
-        cls._patched_environment = None
+        encinfo.install_extra_encodings()
+        polib4us.install_patches()
+        cls._patched_environment = True
 
     def __init__(self, path, *, options):
         if self._patched_environment is not True:
@@ -94,7 +93,7 @@ class Checker(object):
                 self.fake_path = fake_root + path[len(real_root):]
         self.options = options
 
-    @misc.not_overridden
+    @abc.abstractmethod
     def tag(self, tagname, *extra):
         return
 
@@ -167,7 +166,7 @@ class Checker(object):
                 end = broken_encoding.start + 40
                 s = s[begin:end]
                 self.tag('broken-encoding',
-                    tags.safestr(repr(s)[1:]),
+                    s,
                     tags.safestr('cannot be decoded as'),
                     broken_encoding.encoding.upper(),
                 )
@@ -204,7 +203,6 @@ class Checker(object):
                 self.tag('no-language-header-field')
             return
         language = self.options.language
-        linginfo = self.options.linginfo
         language_source = 'command-line'
         language_source_quality = 1
         if language is None:
@@ -216,7 +214,7 @@ class Checker(object):
             if i > 0:
                 language = path_components[i - 1]
                 try:
-                    language = linginfo.parse_language(language)
+                    language = ling.parse_language(language)
                     language.fix_codes()
                     language.remove_encoding()
                     language.remove_nonlinguistic_modifier()
@@ -230,7 +228,7 @@ class Checker(object):
             language, ext = os.path.splitext(os.path.basename(self.path))
             assert ext == '.po'
             try:
-                language = linginfo.parse_language(language)
+                language = ling.parse_language(language)
                 if language.encoding is not None:
                     # It's very likely that something else has been confused
                     # for the apparent encoding.
@@ -245,10 +243,10 @@ class Checker(object):
                 language_source_quality = 0
         if meta_language:
             try:
-                meta_language = linginfo.parse_language(meta_language)
+                meta_language = ling.parse_language(meta_language)
             except ling.LanguageError:
                 try:
-                    new_meta_language = linginfo.get_language_for_name(meta_language)
+                    new_meta_language = ling.get_language_for_name(meta_language)
                 except LookupError:
                     new_meta_language = None
                 if new_meta_language:
@@ -299,7 +297,7 @@ class Checker(object):
             [poedit_language] = poedit_languages
             # FIXME: This should take also X-Poedit-Country into account.
             try:
-                poedit_language = linginfo.get_language_for_name(poedit_language)
+                poedit_language = ling.get_language_for_name(poedit_language)
             except LookupError:
                 self.tag('unknown-poedit-language', poedit_language)
             else:
@@ -485,11 +483,10 @@ class Checker(object):
             content_type_hint = 'text/plain; charset=<encoding>'
             match = re.search(r'(\Atext/plain; )?\bcharset=([^\s;]+)\Z', ct)
             if match:
-                encinfo = self.options.encinfo
                 encoding = match.group(2)
                 try:
-                    is_ascii_compatible = encinfo.is_ascii_compatible_encoding(encoding)
-                except LookupError:
+                    is_ascii_compatible = encinfo.is_ascii_compatible_encoding(encoding, missing_ok=False)
+                except encinfo.EncodingLookupError:
                     if encoding == 'CHARSET':
                         if not is_template:
                             self.tag('boilerplate-in-content-type', ct)
@@ -528,6 +525,11 @@ class Checker(object):
 
     @checks_header_fields('POT-Creation-Date', 'PO-Revision-Date')
     def check_dates(self, file, *, is_template):
+        try:
+            content_type = file.metadata['Content-Type'][0]
+        except IndexError:
+            content_type = ''
+        is_publican = content_type.startswith('application/x-publican;')
         for field in 'POT-Creation-Date', 'PO-Revision-Date':
             dates = file.metadata[field]
             if len(dates) > 1:
@@ -537,14 +539,26 @@ class Checker(object):
                 self.tag('no-date-header-field', field)
                 continue
             for date in dates:
-                if is_template and field.startswith('PO-') and (date == 'YEAR-MO-DA HO:MI+ZONE'):
+                if is_template and field.startswith('PO-') and (date == gettext.boilerplate_date):
                     continue
-                fixed_date = gettext.fix_date_format(date)
-                if fixed_date is None:
+                if 'T' in date and is_publican:
+                    # Publican uses DateTime->now(), which uses the UTC timezone by default:
+                    # http://sources.debian.net/src/publican/2.8-3/lib/Publican/Translate.pm?hl=748#L744
+                    # http://bugs.debian.org/714739
+                    tz_hint = '+0000'
+                else:
+                    tz_hint = None
+                try:
+                    fixed_date = gettext.fix_date_format(date, tz_hint=tz_hint)
+                except gettext.BoilerplateDate:
+                    self.tag('boilerplate-in-date', tags.safestr(field + ':'), date)
+                    continue
+                except gettext.DateSyntaxError:
                     self.tag('invalid-date', tags.safestr(field + ':'), date)
                     continue
-                elif date != fixed_date:
-                    self.tag('invalid-date', tags.safestr(field + ':'), date, '=>', fixed_date)
+                else:
+                    if date != fixed_date:
+                        self.tag('invalid-date', tags.safestr(field + ':'), date, '=>', fixed_date)
                 stamp = gettext.parse_date(fixed_date)
                 if stamp > misc.utc_now():
                     self.tag('date-from-future', tags.safestr(field + ':'), date)
@@ -564,7 +578,7 @@ class Checker(object):
             if project_id_version in {'PACKAGE VERSION', 'PROJECT VERSION'}:
                 self.tag('boilerplate-in-project-id-version', project_id_version)
             else:
-                if not re.search(r'[A-Za-z]', project_id_version):
+                if not re.compile(r'[^_\d\W]', re.UNICODE).search(project_id_version):
                     self.tag('no-package-name-in-project-id-version', project_id_version)
                 if not re.search(r'[0-9]', project_id_version):
                     self.tag('no-version-in-project-id-version', project_id_version)
@@ -583,7 +597,7 @@ class Checker(object):
                 uri = urllib.parse.urlparse(report_msgid_bugs_to)
                 if uri.scheme == '':
                     self.tag('invalid-report-msgid-bugs-to', report_msgid_bugs_to)
-            elif rfc2606.is_reserved_email(email_address):
+            elif domains.is_email_in_special_domain(email_address):
                 self.tag('invalid-report-msgid-bugs-to', report_msgid_bugs_to)
             elif email_address == 'EMAIL@ADDRESS':
                 self.tag('boilerplate-in-report-msgid-bugs-to', report_msgid_bugs_to)
@@ -603,7 +617,7 @@ class Checker(object):
             translator_emails.add(translator_email)
             if '@' not in translator_email:
                 self.tag('invalid-last-translator', translator)
-            elif rfc2606.is_reserved_email(translator_email):
+            elif domains.is_email_in_special_domain(translator_email):
                 self.tag('invalid-last-translator', translator)
             elif translator_email == 'EMAIL@ADDRESS':
                 if not is_template:
@@ -621,7 +635,7 @@ class Checker(object):
                 # TODO: An URL is also allowed here.
                 # self.tag('invalid-language-team', translator)
                 pass
-            elif rfc2606.is_reserved_email(team_email):
+            elif domains.is_email_in_special_domain(team_email):
                 self.tag('invalid-language-team', team)
             elif team_email in {'LL@li.org', 'EMAIL@ADDRESS'}:
                 if not is_template:
@@ -642,7 +656,9 @@ class Checker(object):
                 self.tag('duplicate-header-entry')
                 break
             if entry.occurrences:
-                self.tag('empty-msgid-message-with-source-code-references')
+                self.tag('empty-msgid-message-with-source-code-references',
+                    *(':'.join((path, line)) for path, line in entry.occurrences)
+                )
             if entry.msgid_plural or entry.msgstr_plural:
                 self.tag('empty-msgid-message-with-plural-forms')
             try:
@@ -671,7 +687,6 @@ class Checker(object):
                 self.tag('distant-header-entry')
             unusual_chars = set(find_unusual_characters(msgstr))
             if unusual_chars:
-                encinfo = self.options.encinfo
                 unusual_char_names = ', '.join(
                     'U+{:04X} {}'.format(ord(ch), encinfo.get_character_name(ch))
                     for ch in sorted(unusual_chars)
@@ -686,7 +701,7 @@ class Checker(object):
                     seen_conflict_marker = True
             else:
                 self.tag('stray-header-line', stray)
-        header_fields = frozenset(self.options.gettextinfo.header_fields)
+        header_fields = frozenset(gettext.header_fields)
         header_fields_lc = {str.lower(s): s for s in header_fields}
         for key, values in sorted(metadata.items()):
             if not key.startswith('X-'):
@@ -710,7 +725,6 @@ class Checker(object):
     def check_messages(self, file, *, encoding):
         if encoding is None:
             return
-        encinfo = self.options.encinfo
         found_unusual_characters = set()
         msgid_counter = collections.Counter()
         messages = [msg for msg in file if not is_header_entry(msg)]
@@ -719,8 +733,101 @@ class Checker(object):
                 continue
             if is_header_entry(message):
                 continue
-            flags = resplit_flags(message.flags)
-            fuzzy = 'fuzzy' in flags
+            flags = collections.Counter(resplit_flags(message.flags))
+            fuzzy = False
+            wrap = None
+            format_flags = collections.defaultdict(dict)
+            for flag, n in sorted(flags.items()):
+                known_flag = True
+                if flag == 'fuzzy':
+                    fuzzy = True
+                elif flag in {'wrap', 'no-wrap'}:
+                    new_wrap = flag == 'wrap'
+                    if wrap == (not new_wrap):
+                        self.tag('conflicting-message-flags',
+                            message_repr(message, template='{}:'),
+                            'wrap', 'no-wrap'
+                        )
+                    else:
+                        wrap = new_wrap
+                elif flag.startswith('range:'):
+                    if not message.msgid_plural:
+                        self.tag('range-flag-without-plural-string')
+                    match = re.match('([0-9]+)[.][.]([0-9]+)', flag[6:].strip(' \t\r\f\v'))
+                    if match is not None:
+                        i, j = map(int, match.groups())
+                        if i >= j:
+                            match = None
+                    if match is None:
+                        self.tag('invalid-range-flag',
+                            message_repr(message, template='{}:'),
+                            flag
+                        )
+                elif flag.endswith('-format'):
+                    known_flag = False
+                    for prefix in 'no-', 'possible-', 'impossible-', '':
+                        tp = prefix.rstrip('-')
+                        if not flag.startswith(prefix):
+                            continue
+                        string_format = flag[len(prefix):-7]
+                        if string_format in gettext.string_formats:
+                            known_flag = True
+                            format_flags[tp][string_format] = flag
+                            break
+                else:
+                    known_flag = False
+                if not known_flag:
+                    self.tag('unknown-message-flag',
+                        message_repr(message, template='{}:'),
+                        flag
+                    )
+                if n > 1 and flag:
+                    self.tag('duplicate-message-flag',
+                        message_repr(message, template='{}:'),
+                        flag
+                    )
+            positive_format_flags = format_flags['']
+            if len(positive_format_flags) > 1:
+                self.tag('conflicting-message-flags',
+                    message_repr(message, template='{}:'),
+                    *sorted(positive_format_flags.values())
+                )
+            elif len(positive_format_flags) == 1:
+                [[positive_format, positive_format_flag]] = positive_format_flags.items()
+                negative_format_flags = sorted(
+                    format_flag
+                    for fmt, format_flag in
+                    format_flags['no'].items()
+                    if fmt != positive_format
+                )
+                if negative_format_flags:
+                    args = (
+                        negative_format_flags +
+                        [tags.safe_format('(implied by {flag})'.format(flag=positive_format_flag))]
+                    )
+                    self.tag('redundant-message-flag',
+                        message_repr(message, template='{}:'),
+                        *args
+                    )
+            for positive_key, negative_key in [('', 'no'), ('', 'impossible'), ('possible', 'impossible')]:
+                positive_format_flags = format_flags[positive_key]
+                negative_format_flags = format_flags[negative_key]
+                conflicting_formats = frozenset(positive_format_flags) & frozenset(negative_format_flags)
+                for fmt in sorted(conflicting_formats):
+                    self.tag('conflicting-message-flags',
+                        message_repr(message, template='{}:'),
+                        positive_format_flags[fmt],
+                        negative_format_flags[fmt],
+                    )
+            positive_format_flags = format_flags['']
+            possible_format_flags = format_flags['possible']
+            redundant_formats = frozenset(positive_format_flags) & frozenset(possible_format_flags)
+            for fmt in sorted(redundant_formats):
+                self.tag('redundant-message-flag',
+                    message_repr(message, template='{}:'),
+                    possible_format_flags[fmt],
+                    tags.safe_format('(implied by {flag})'.format(flag=positive_format_flags[fmt]))
+                )
             leading_lf = message.msgid.startswith('\n')
             trailing_lf = message.msgid.endswith('\n')
             strings = [message.msgid_plural]
@@ -761,7 +868,13 @@ class Checker(object):
             if msgid_counter[message.msgid, message.msgctxt] == 2:
                 self.tag('duplicate-message-definition', message_repr(message))
         if len(msgid_counter) == 0:
-            self.tag('empty-file')
+            possible_hidden_strings = False
+            if isinstance(file, polib.MOFile):
+                possible_hidden_strings = file.possible_hidden_strings
+            if not possible_hidden_strings:
+                self.tag('empty-file')
+
+__all__ = ['Checker']
 
 def is_header_entry(entry):
     return (
@@ -771,7 +884,7 @@ def is_header_entry(entry):
 
 def resplit_flags(flags):
     return (
-        flag.strip('\t\r\f\v')
+        flag.strip(' \t\r\f\v')
         for subflags in flags
         for flag in subflags.split(',')
     ) # work-around for https://bitbucket.org/izi/polib/issue/46
