@@ -1,4 +1,4 @@
-# Copyright © 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2012, 2013, 2014 Jakub Wilk <jwilk@jwilk.net>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the “Software”), to deal
@@ -18,6 +18,10 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
+'''
+command-line interface
+'''
+
 import argparse
 import io
 import os
@@ -27,14 +31,13 @@ import sys
 import tempfile
 
 from . import checker
-from . import encodings
-from . import gettext
 from . import ling
 from . import misc
+from . import paths
 from . import tags
 from . import terminal
 
-__version__ = '0.11.1'
+__version__ = '0.13.4'
 
 def initialize_terminal():
     if sys.stdout.isatty():
@@ -52,7 +55,7 @@ class Checker(checker.Checker):
         if tagname in self.options.ignore_tags:
             return
         try:
-            tag = self.options.taginfo[tagname]
+            tag = tags.get_tag(tagname)
         except KeyError:
             raise misc.DataIntegrityError(
                 'attempted to emit an unknown tag: {tag!r}'.format(tag=tagname)
@@ -107,7 +110,18 @@ def check_deb(filename, *, options):
     finally:
         shutil.rmtree(tmpdir)
 
-def main(basedir):
+def check_all(files, *, options):
+    for filename in files:
+        if options.debian:
+            try:
+                check_deb(filename, options=options)
+            except UnsupportedFileType:
+                pass
+            else:
+                continue
+        check_file(filename, options=options)
+
+def main():
     initialize_terminal()
     is_like_debian = misc.OSRelease().is_like('debian')
     ap = argparse.ArgumentParser(description=__doc__)
@@ -120,18 +134,10 @@ def main(basedir):
     options = ap.parse_args()
     files = options.files
     del options.files
-    datadir = os.path.join(
-        basedir,
-        'data', ''
-    )
-    os.stat(datadir)
-    options.encinfo = encinfo = encodings.EncodingInfo(datadir)
-    options.gettextinfo = gettext.GettextInfo(datadir)
-    options.linginfo = linginfo = ling.LingInfo(datadir)
-    options.taginfo = tags.TagInfo(datadir)
+    paths.check()
     if options.language is not None:
         try:
-            language = linginfo.parse_language(options.language)
+            language = ling.parse_language(options.language)
             language.fix_codes()
         except ling.LanguageError:
             if options.traceback:
@@ -142,18 +148,9 @@ def main(basedir):
         options.language = language
     options.ignore_tags = set()
     options.fake_root = None
-    with Checker.patched_environment(encinfo):
-        check_all(files, options=options)
+    Checker.patch_environment()
+    check_all(files, options=options)
 
-def check_all(files, *, options):
-    for filename in files:
-        if options.debian:
-            try:
-                check_deb(filename, options=options)
-            except UnsupportedFileType:
-                pass
-            else:
-                continue
-        check_file(filename, options=options)
+__all__ = ['main']
 
 # vim:ts=4 sw=4 et
