@@ -24,9 +24,7 @@ miscellanea
 
 import datetime
 import errno
-import functools
 import shlex
-import warnings
 
 def is_sorted(iterable):
     # It's not very efficient, but should be enough for our purposes.
@@ -41,25 +39,6 @@ def check_sorted(iterable, exception=DataIntegrityError):
     if not is_sorted(iterable):
         raise exception()
 
-class NotOverriddenWarning(UserWarning):
-    pass
-
-def not_overridden(f):
-    '''
-    Issue NotOverriddenWarning if the decorated method was not overridden in a
-    subclass, or called directly.
-    '''
-    @functools.wraps(f)
-    def new_f(self, *args, **kwargs):
-        cls = type(self)
-        warnings.warn(
-            '%s.%s.%s() is not overridden' % (cls.__module__, cls.__name__, f.__name__),
-            category=NotOverriddenWarning,
-            stacklevel=2
-        )
-        return f(self, *args, **kwargs)
-    return new_f
-
 class OSRelease(object):
 
     '''
