@@ -1,4 +1,4 @@
-# Copyright © 2013 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2013, 2014 Jakub Wilk <jwilk@jwilk.net>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the “Software”), to deal
@@ -22,13 +22,20 @@
 MO file parser
 '''
 
+# MO file format documentation:
+# * http://git.savannah.gnu.org/cgit/gettext.git/tree/gettext-runtime/intl/gmo.h?id=v0.18.3
+# * http://git.savannah.gnu.org/cgit/gettext.git/tree/gettext-tools/src/read-mo.c?id=v0.18.3
+# * http://www.gnu.org/software/gettext/manual/html_node/MO-Files.html
+
 import re
 import struct
 import sys
 
 import polib
 
-little_endian_magic = b'\xde\x12\x04\x95'
+from . import encodings
+
+little_endian_magic = b'\xDE\x12\x04\x95'
 big_endian_magic = little_endian_magic[::-1]
 
 class SyntaxError(Exception):
@@ -45,7 +52,7 @@ class Parser(object):
         view = memoryview(contents)
         if len(view) > 0:
             if sys.version_info >= (3, 3):
-                # http://docs.python.org/dev/whatsnew/3.3.html#pep-3118-new-memoryview-implementation-and-buffer-protocol-documentation
+                # http://docs.python.org/3.3/whatsnew/3.3.html#pep-3118-new-memoryview-implementation-and-buffer-protocol-documentation
                 # “Accessing a memoryview element with format ‘B’ (unsigned bytes)
                 # now returns an integer […]. For returning a bytes object the view
                 # must be cast to ‘c’ first.”
@@ -91,6 +98,16 @@ class Parser(object):
         if major_revision > 1:
             raise SyntaxError('unexpected major revision number: {n}'.format(n=major_revision))
         [n_strings] = self._read_ints(at=8)
+        possible_hidden_strings = False
+        if minor_revision > 1:
+            # “an unexpected minor revision number means that the file can be
+            # read but will not reveal its full contents”
+            possible_hidden_strings = True
+        elif minor_revision == 1:
+            [n_sysdep_strings] = self._read_ints(at=36)
+            if n_sysdep_strings > 0:
+                possible_hidden_strings = True
+        self.instance.possible_hidden_strings = possible_hidden_strings
         [msgid_offset, msgstr_offset] = self._read_ints(at=12, n=2)
         self._last_msgid = None
         for i in range(n_strings):
@@ -123,7 +140,7 @@ class Parser(object):
         encoding = self._encoding
         if i == 0:
             if encoding is None and msgid == b'':
-                # http://git.savannah.gnu.org/cgit/gettext.git/tree/gettext-runtime/intl/dcigettext.c?id=159cd1aefcf2#n1106
+                # http://git.savannah.gnu.org/cgit/gettext.git/tree/gettext-runtime/intl/dcigettext.c?id=v0.18.3#n1106
                 match = re.search(b'charset=([^ \t\n]+)', msgstr)
                 if match is not None:
                     try:
@@ -133,9 +150,7 @@ class Parser(object):
             if encoding is None:
                 encoding = 'ASCII'
             else:
-                try:
-                    b'charset'.decode(encoding)
-                except (LookupError, UnicodeError):
+                if not encodings.is_ascii_compatible_encoding(encoding):
                     encoding = 'ASCII'
             self._encoding = encoding
         else:
@@ -179,5 +194,7 @@ def main():
 
 if __name__ == '__main__':
     main()
+else:
+    del main
 
 # vim:ts=4 sw=4 et
