@@ -1,4 +1,4 @@
-# Copyright © 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2012, 2013, 2014 Jakub Wilk <jwilk@jwilk.net>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the “Software”), to deal
@@ -30,27 +30,16 @@ import re
 
 import polib
 
+from . import encodings
 from . import moparser
 
 patches = []
 
-@contextlib.contextmanager
-def patched(*, patches=patches):
-    '''
-    with polib4us.patched():
-        ...
-    '''
-    patches = tuple(patches)
-    try:
-        current_patch, *remaining_patches = patches
-    except ValueError:
-        yield
-        return
-    with current_patch():
-        with patched(patches=remaining_patches):
-            yield
+def install_patches(patches=patches):
+    for patch in patches:
+        patch()
 
-__all__ = ['patched']
+__all__ = ['install_patches']
 
 def register_patch(patch):
     patches.append(contextlib.contextmanager(patch))
@@ -62,42 +51,51 @@ def register_patch(patch):
 
 @register_patch
 def default_encoding_patch():
-    original = polib.default_encoding
-    try:
-        polib.default_encoding = 'ASCII'
-        yield
-    finally:
-        polib.default_encoding = original
+    polib.default_encoding = 'ASCII'
 
 # polib.codecs
 # ============
-# Work around a newline decoding bug:
-# <http://bugs.debian.org/692283>.
+# Work around a few PO parsing bugs:
+# - newline decoding: http://bugs.debian.org/692283
+# - trailing comment parsing: https://bitbucket.org/izi/polib/issue/51
+# - atypical comment parsing
+# - parsing of empty files: https://bitbucket.org/izi/polib/issue/59
 
 class Codecs(object):
 
     _iterlines = re.compile(r'[^\n]*(?:\n|\Z)').findall
+    _atypical_comment = re.compile(r'#[^ .:,|~]').match
 
     def __getattr__(self, attr):
         return getattr(codecs, attr)
 
     def open(self, path, mode, encoding):
-        if mode != 'rU':
+        if mode not in {'rU', 'rt'}:
             raise NotImplementedError
+        if not encodings.is_ascii_compatible_encoding(encoding):
+            encoding = 'ASCII'
         with open(path, 'rb') as file:
             contents = file.read()
         contents = contents.decode(encoding)
+        pending_comments = []
+        empty = True
         for line in self._iterlines(contents):
-            yield line
+            if self._atypical_comment(line):
+                line = '# ' + line[1:]
+            if line[:2] in {'', '# '} or line.isspace():
+                pending_comments += [line]
+            else:
+                for comment_line in pending_comments:
+                    yield comment_line
+                pending_comments = []
+                yield line
+                empty = False
+        if empty:
+            yield '# '
 
 @register_patch
 def codecs_patch():
-    original = polib.codecs
-    try:
-        polib.codecs = Codecs()
-        yield
-    finally:
-        polib.codecs = original
+    polib.codecs = polib.io = Codecs()
 
 # polib.POFile.find()
 # ===================
@@ -107,14 +105,9 @@ def codecs_patch():
 
 @register_patch
 def pofile_find_patch():
-    original = polib.POFile.find
     def pofile_find(self, *args, **kwargs):
         return
-    try:
-        polib.POFile.find = pofile_find
-        yield
-    finally:
-        polib.POFile.find = original
+    polib.POFile.find = pofile_find
 
 # polib.unescape()
 # ================
@@ -151,12 +144,7 @@ def polib_unescape(s):
 
 @register_patch
 def unescape_patch():
-    original = polib.unescape
-    try:
-        polib.unescape = polib_unescape
-        yield
-    finally:
-        polib.unescape = original
+    polib.unescape = polib_unescape
 
 # polib._MOFileParser
 # ===================
@@ -168,12 +156,7 @@ def unescape_patch():
 
 @register_patch
 def mo_parser_patch():
-    original = polib._MOFileParser
-    try:
-        polib._MOFileParser = moparser.Parser
-        yield
-    finally:
-        polib._MOFileParser = original
+    polib._MOFileParser = moparser.Parser
 
 # polib.detect_encoding()
 # =======================
@@ -187,10 +170,6 @@ def detect_encoding_patch():
             return
         return original(path)
     original = polib.detect_encoding
-    try:
-        polib.detect_encoding = detect_encoding
-        yield
-    finally:
-        polib.detect_encoding = original
+    polib.detect_encoding = detect_encoding
 
 # vim:ts=4 sw=4 et
