@@ -1,4 +1,4 @@
-# Copyright © 2012 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the “Software”), to deal
@@ -28,6 +28,7 @@ import os
 import re
 
 from . import misc
+from . import paths
 from . import terminal
 
 @functools.total_ordering
@@ -87,6 +88,15 @@ certainties = OrderedGroup('Certainty',
     'certain',
 )
 
+class InvalidSeverity(misc.DataIntegrityError):
+    pass
+
+class InvalidCertainty(misc.DataIntegrityError):
+    pass
+
+class UnknownField(misc.DataIntegrityError):
+    pass
+
 _is_safe = re.compile(r'\A[A-Za-z0-9_.!<>=-]+\Z').match
 
 class safestr(str):
@@ -95,6 +105,8 @@ class safestr(str):
 def _escape(s):
     if isinstance(s, safestr):
         return s
+    if isinstance(s, bytes):
+        return repr(s)[1:]
     s = str(s)
     if s == '':
         return '(empty string)'
@@ -117,17 +129,25 @@ class Tag(object):
             try:
                 getattr(self, '_set_' + k)(v)
             except AttributeError:
-                raise KeyError(k)
+                raise UnknownField(k)
         self.name, self.severity, self.certainty
 
     def _set_name(self, value):
         self.name = value
 
     def _set_severity(self, value):
-        self.severity = severities[value]
+        try:
+            self.severity = severities[value]
+        except KeyError as exc:
+            [key] = exc.args
+            raise InvalidSeverity(key)
 
     def _set_certainty(self, value):
-        self.certainty = certainties[value]
+        try:
+            self.certainty = certainties[value]
+        except KeyError as exc:
+            [key] = exc.args
+            raise InvalidCertainty(key)
 
     _strip_leading_dot = functools.partial(
         re.compile('^[.]', re.MULTILINE).sub,
@@ -191,32 +211,29 @@ class Tag(object):
             s += ' ' + ' '.join(map(_escape, extra))
         return s
 
-class TagInfo(object):
+def _read_tags():
+    path = os.path.join(paths.datadir, 'tags')
+    cp = configparser.ConfigParser(interpolation=None, default_section='')
+    cp.read(path, encoding='UTF-8')
+    misc.check_sorted(cp)
+    tags = {}
+    for tagname, section in cp.items():
+        if not tagname:
+            continue
+        kwargs = dict(section.items())
+        kwargs['name'] = tagname
+        tags[tagname] = Tag(**kwargs)
+    return tags
 
-    def __init__(self, datadir):
-        path = os.path.join(datadir, 'tags')
-        cp = configparser.ConfigParser(interpolation=None, default_section='')
-        cp.read(path, encoding='UTF-8')
-        misc.check_sorted(cp)
-        self._tags = {}
-        for tagname, section in cp.items():
-            if not tagname:
-                continue
-            kwargs = dict(section.items())
-            kwargs['name'] = tagname
-            try:
-                self._tags[tagname] = Tag(**kwargs)
-            except KeyError as exc:
-                [key] = exc.args
-                raise misc.DataIntegrityError('unknown field: {!r}'.format(key))
+_tags = _read_tags()
 
-    def __contains__(self, tagname):
-        return tagname in self._tags
+def tag_exists(tagname):
+    return tagname in _tags
 
-    def __iter__(self):
-        return (tag for _, tag in sorted(self._tags.items()))
+def iter_tags():
+    return (tag for _, tag in sorted(_tags.items()))
 
-    def __getitem__(self, tagname):
-        return self._tags[tagname]
+def get_tag(tagname):
+    return _tags[tagname]
 
 # vim:ts=4 sw=4 et
