@@ -19,6 +19,7 @@
 # SOFTWARE.
 
 import difflib
+import errno
 import inspect
 import os
 import re
@@ -45,7 +46,7 @@ def this():
 
 # ----------------------------------------
 
-_parse_etag = re.compile('# ([A-Z]): (([\w-]+).*)').match
+_parse_etag = re.compile('([A-Z]): (([\w-]+).*)').match
 
 def parse_etag(contents, path, multi_line=False):
     match = _parse_etag(contents)
@@ -154,7 +155,10 @@ class Plugin(nose.plugins.Plugin):
         yield TestCase(path)
 
     def wantFunction(self, func):
-        if func is test_file:
+        # If the plugin is being used, test_file() is reduntant.
+        # We can't just check whether "func is test_file", because some
+        # versions of nose (at least 1.1.2) reload the module.
+        if func.__name__ == 'test_file' and func.__module__ == test_file.__module__:
             return False
 
 class TestCase(unittest.TestCase):
@@ -203,7 +207,10 @@ def assert_emit_tags(path, etags, *, options=()):
         else:
             message += ['stderr: (empty)']
         raise AssertionError('\n'.join(message))
+    expected_failure = os.path.basename(path).startswith('xfail-')
     if stdout != etags:
+        if expected_failure:
+            raise nose.SkipTest('expected failure')
         str_etags = [str(x) for x in etags]
         message = ['Tags differ:', '']
         diff = list(
@@ -211,33 +218,68 @@ def assert_emit_tags(path, etags, *, options=()):
         )
         message += diff[3:]
         raise AssertionError('\n'.join(message))
+    elif expected_failure:
+        raise AssertionError('unexpected success')
+
+class TestFileSyntaxError(Exception):
+    pass
+
+def _parse_test_header_file(file, path, *, comments_only):
+    etags = []
+    options = []
+    for n, line in enumerate(file):
+        orig_line = line
+        if comments_only:
+            if n == 0 and line.startswith('#!'):
+                continue
+            if line.startswith('# '):
+                line = line[2:]
+            else:
+                break
+        if line.startswith('--'):
+            options += shlex.split(line)
+        else:
+            etag = parse_etag(line, path)
+            if etag is None:
+                if comments_only:
+                    break
+                else:
+                    raise TestFileSyntaxError(orig_line)
+            etags += [etag]
+    return etags, options
+
+def _parse_test_headers(path):
+    # <path>.tags:
+    try:
+        file = open(path + '.tags', encoding='ASCII')
+    except IOError as exc:
+        if exc.errno != errno.ENOENT:
+            raise
+    else:
+        with file:
+            return _parse_test_header_file(file, path, comments_only=False)
+    # <path>.gen:
+    try:
+        file = open(path + '.gen', encoding='ASCII', errors='ignore')
+    except IOError as exc:
+        if exc.errno != errno.ENOENT:
+            raise
+    else:
+        with file:
+            return _parse_test_header_file(file, path, comments_only=True)
+    # <path>:
+    with open(path, 'rt', encoding='ASCII', errors='ignore') as file:
+        return _parse_test_header_file(file, path, comments_only=True)
 
 def _test_file(path):
     path = os.path.relpath(os.path.join(here, path), start=os.getcwd())
     options = []
-    etags = []
-    with open(path, 'rt', encoding='ASCII', errors='ignore') as file:
-        for line in file:
-            if not line.startswith('# '):
-                break
-            etag = parse_etag(line, path)
-            if etag is None:
-                if line.startswith('# --'):
-                    options += shlex.split(line[2:])
-                    continue
-                break
-            etags += [etag]
+    etags, options = _parse_test_headers(path)
     assert_emit_tags(path, etags, options=options)
 
 def get_coverage_for_file(path):
-    with open(path, 'rt', encoding='ASCII', errors='ignore') as file:
-        for line in file:
-            if not line.startswith('# '):
-                break
-            etag = parse_etag(line, '')
-            if etag is None:
-                continue
-            yield etag.tag
+    etags, options = _parse_test_headers(path)
+    return (t.tag for t in etags)
 
 def get_coverage_for_function(fn):
     for etag in etags_from_tagstring(fn, ''):
@@ -256,7 +298,7 @@ def test_file():
         yield _test_file, path
 
 @tagstring('''
-# E: os-error No such file or directory
+E: os-error No such file or directory
 ''')
 def test_os_error_no_such_file():
     with aux.temporary_directory() as tmpdir:
@@ -265,7 +307,7 @@ def test_os_error_no_such_file():
         assert_emit_tags(path, expected)
 
 @tagstring('''
-# E: os-error Permission denied
+E: os-error Permission denied
 ''')
 def test_os_error_permission_denied():
     if os.getuid() == 0:
@@ -278,17 +320,6 @@ def test_os_error_permission_denied():
         expected = etags_from_tagstring(this(), path)
         assert_emit_tags(path, expected)
 
-@tagstring('''
-# E: invalid-mo-file unexpected magic
-''')
-def test_empty_mo_file():
-    with aux.temporary_directory() as tmpdir:
-        path = os.path.join(tmpdir, 'empty-mo-file.mo')
-        with open(path, 'wb'):
-            pass
-        expected = etags_from_tagstring(this(), path)
-        assert_emit_tags(path, expected)
-
 # ----------------------------------------
 
 def get_coverage():
