@@ -1,4 +1,4 @@
-# Copyright © 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2012, 2013, 2014 Jakub Wilk <jwilk@jwilk.net>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the “Software”), to deal
@@ -19,7 +19,9 @@
 # SOFTWARE.
 
 import curses.ascii
+import sys
 
+import nose
 from nose.tools import (
     assert_equal,
     assert_false,
@@ -31,9 +33,7 @@ from nose.tools import (
 
 from . import aux
 
-import lib.encodings
-
-E = lib.encodings.EncodingInfo(aux.datadir)
+import lib.encodings as E
 
 class test_is_portable_encoding:
 
@@ -73,21 +73,37 @@ class test_propose_portable_encoding:
 class test_ascii_compatiblity:
 
     def test_portable(self):
-
         def _test(encoding):
             assert_true(E.is_ascii_compatible_encoding(encoding))
-
+            assert_true(E.is_ascii_compatible_encoding(encoding, missing_ok=False))
         for encoding in E.get_portable_encodings():
             yield _test, encoding
 
     def test_incompatible(self):
-
         def _test(encoding):
             assert_false(E.is_ascii_compatible_encoding(encoding))
-
+            assert_false(E.is_ascii_compatible_encoding(encoding, missing_ok=False))
         yield _test, 'UTF-7'
         yield _test, 'UTF-16'
 
+    def _test_missing(self, encoding):
+        assert_false(E.is_ascii_compatible_encoding(encoding))
+        with assert_raises(E.EncodingLookupError):
+            E.is_ascii_compatible_encoding(encoding, missing_ok=False)
+
+    def test_non_text(self):
+        _test = self._test_missing
+        yield _test, 'base64_codec'
+        yield _test, 'bz2_codec'
+        yield _test, 'hex_codec'
+        yield _test, 'quopri_codec'
+        yield _test, 'rot_13'
+        yield _test, 'uu_codec'
+        yield _test, 'zlib_codec'
+
+    def test_missing(self):
+        self._test_missing('eggs')
+
 class test_get_character_name:
 
     def test_latin(self):
@@ -125,68 +141,128 @@ class test_get_character_name:
             names.add(name)
 
     def test_non_character(self):
-        name = E.get_character_name('\ufffe')
+        name = E.get_character_name('\uFFFE')
         assert_equal(name, 'non-character')
-        name = E.get_character_name('\uffff')
+        name = E.get_character_name('\uFFFF')
         assert_equal(name, 'non-character')
 
     def test_lookup_error(self):
         with assert_raises(ValueError):
-            E.get_character_name('\ue000')
+            E.get_character_name('\uE000')
 
 class test_extra_encoding:
 
+    @aux.fork_isolation
     def test_install(self):
+        encoding = 'VISCII'
         def enc():
-            ''.encode('VISCII')
+            ''.encode(encoding)
         def dec():
-            b'.'.decode('VISCII')
-        with assert_raises(LookupError):
-            enc()
-        with assert_raises(LookupError):
-            dec()
-        with E.extra_encodings():
-            enc()
-            dec()
-        with assert_raises(LookupError):
+            b'.'.decode(encoding)
+        try:
             enc()
+        except LookupError:
+            pass
+        else:
+            raise nose.SkipTest(
+                'python{ver[0]}.{ver[1]} supports the {enc} encoding'.format(
+                    ver=sys.version_info,
+                    enc=encoding
+                )
+            )
         with assert_raises(LookupError):
             dec()
+        E.install_extra_encodings()
+        enc()
+        dec()
 
+    @aux.fork_isolation
+    def test_8859(self):
+        E.install_extra_encodings()
+        encoding = '8859-2'
+        portable_encoding = E.propose_portable_encoding(encoding)
+        assert_equal('ISO-' + encoding, portable_encoding)
+
+    @aux.fork_isolation
     def test_not_allowed(self):
+        encoding = 'TSCII'
         def enc():
-            ''.encode('RUSCII')
+            ''.encode(encoding)
+        try:
+            enc()
+        except LookupError:
+            pass
+        else:
+            raise nose.SkipTest(
+                'python{ver[0]}.{ver[1]} supports the {enc} encoding'.format(
+                    ver=sys.version_info,
+                    enc=encoding
+                )
+            )
+        E.install_extra_encodings()
         with assert_raises(LookupError):
             enc()
-        with E.extra_encodings():
-            with assert_raises(LookupError):
-                enc()
 
     _viscii_unicode = 'Ti\u1EBFng Vi\u1EC7t'
-    _viscii_bytes = b'Ti\xaang Vi\xaet'
+    _viscii_bytes = b'Ti\xAAng Vi\xAEt'
 
-    def test_encode(self):
+    @aux.fork_isolation
+    def test_8b_encode(self):
+        E.install_extra_encodings()
         u = self._viscii_unicode
-        with E.extra_encodings():
-            b = u.encode('VISCII')
+        b = u.encode('VISCII')
         assert_equal(b, self._viscii_bytes)
 
-    def test_encode_error(self):
+    @aux.fork_isolation
+    def test_8b_encode_error(self):
+        E.install_extra_encodings()
         u = self._viscii_unicode
-        with E.extra_encodings():
-            with assert_raises(UnicodeEncodeError):
-                u.encode('KOI8-RU')
+        with assert_raises(UnicodeEncodeError):
+            u.encode('KOI8-RU')
 
-    def test_decode(self):
+    @aux.fork_isolation
+    def test_8b_decode(self):
+        E.install_extra_encodings()
         b = self._viscii_bytes
-        with E.extra_encodings():
-            u = b.decode('VISCII')
+        u = b.decode('VISCII')
         assert_equal(u, self._viscii_unicode)
 
-    def test_decode_error(self):
+    @aux.fork_isolation
+    def test_8b_decode_error(self):
+        E.install_extra_encodings()
         b = self._viscii_bytes
-        with E.extra_encodings():
-            with assert_raises(UnicodeDecodeError):
-                b.decode('EUC-TW')
+        with assert_raises(UnicodeDecodeError):
+            b.decode('KOI8-T')
+
+    _euc_tw_unicode = '\u4E2D\u6587'
+    _euc_tw_bytes = b'\xC4\xE3\xC5\xC6'
+
+    @aux.fork_isolation
+    def test_mb_encode(self):
+        E.install_extra_encodings()
+        u = self._euc_tw_unicode
+        b = u.encode('EUC-TW')
+        assert_equal(b, self._euc_tw_bytes)
+
+    @aux.fork_isolation
+    def test_mb_encode_error(self):
+        E.install_extra_encodings()
+        u = self._viscii_unicode
+        with assert_raises(UnicodeEncodeError):
+            u.encode('EUC-TW')
+
+    @aux.fork_isolation
+    def test_mb_decode(self):
+        E.install_extra_encodings()
+        b = self._euc_tw_bytes
+        u = b.decode('EUC-TW')
+        assert_equal(u, self._euc_tw_unicode)
+
+    @aux.fork_isolation
+    def test_mb_decode_error(self):
+        E.install_extra_encodings()
+        b = self._viscii_bytes
+        with assert_raises(UnicodeDecodeError):
+            b.decode('EUC-TW')
 
 # vim:ts=4 sw=4 et
